@@ -10,7 +10,9 @@ class TestArchive < Test::Unit::TestCase
   end
 
   def tempfile
-    Tempfile.new('archive-test').path
+    tempfile_object = Tempfile.new('archive-test')
+    tempfile_object.close # close to avoid locking from git processes
+    tempfile_object.path
   end
 
   def test_archive
@@ -26,9 +28,10 @@ class TestArchive < Test::Unit::TestCase
     f = @git.object('v2.6').archive(nil, :format => 'tar') # returns path to temp file
     assert(File.exist?(f))
 
-    lines = `cd /tmp; tar xvpf #{f} 2>&1`.split("\n")
-    assert_match(%r{ex_dir/}, lines[0])
-    assert_match(/example.txt/, lines[2])
+    lines = Minitar::Input.open(f).each.to_a.map(&:full_name)
+    assert_match(%r{ex_dir/}, lines[1])
+    assert_match(/ex_dir\/ex\.txt/, lines[2])
+    assert_match(/example\.txt/, lines[3])
 
     f = @git.object('v2.6').archive(tempfile, :format => 'zip')
     assert(File.file?(f))
@@ -36,17 +39,21 @@ class TestArchive < Test::Unit::TestCase
     f = @git.object('v2.6').archive(tempfile, :format => 'tgz', :prefix => 'test/')
     assert(File.exist?(f))
 
+    lines = Minitar::Input.open(Zlib::GzipReader.new(File.open(f, 'rb'))).each.to_a.map(&:full_name)
+    assert_match(%r{test/}, lines[1])
+    assert_match(%r{test/ex_dir/ex\.txt}, lines[3])
+
     f = @git.object('v2.6').archive(tempfile, :format => 'tar', :prefix => 'test/', :path => 'ex_dir/')
     assert(File.exist?(f))
-
-    lines = `cd /tmp; tar xvpf #{f} 2>&1`.split("\n")
-    assert_match(%r{test/}, lines[0])
-    assert_match(%r{test/ex_dir/ex\.txt}, lines[2])
+ 
+    lines = Minitar::Input.open(f).each.to_a.map(&:full_name)
+    assert_match(%r{test/}, lines[1])
+    assert_match(%r{test/ex_dir/ex\.txt}, lines[3])
 
     in_temp_dir do
       c = Git.clone(@wbare, 'new')
       c.chdir do
-        f = @git.remote('origin').branch('master').archive(tempfile, :format => 'tgz')
+        f = @git.remote('working').branch('master').archive(tempfile, :format => 'tgz')
         assert(File.exist?(f))
       end
     end
